<?php

namespace Drupal\mcapi_bursers\Plugin\EntityReferenceSelection;

use Drupal\mcapi\Plugin\EntityReferenceSelection\WalletSelection;

/**
 * Provide default Wallet selection handler.
 *
 * The settings consist of one value, restriction, which can be in 3 modes
 *
 * @EntityReferenceSelection(
 *   id = "default:mcapi_wallet_burser",
 *   label = @Translation("Wallet (included bursered) selection"),
 *   entity_types = {"mcapi_wallet"},
 *   group = "default"
 * )
 */
class WalletBurserSelection extends WalletSelection {

  /**
   * {@inheritdoc}
   */
  public function buildEntityQuery($match = NULL, $match_operator = 'CONTAINS') {
    $query = $this->entityTypeManager->getStorage('mcapi_wallet')->getQuery()->accessCheck(TRUE);
    if ($match) {
      $query->condition('name', $match, $match_operator);
    }
    switch ($this->configuration['restriction']) {
      case static::RESTRICTION_MINE:
        $query->condition('holder.target_type', 'user');
        $db_or = $query->orConditionGroup();
        $db_or->condition('holder.target_id', $this->currentUser->id(), 'IN')
          ->condition('bursers', $this->currentUser->id());
        $query->condition($db_or);
        break;
      case static::RESTRICTION_NOTMINE:
        $query->condition('holder.target_type', 'user')
          ->condition('holder.target_id', $this->currentUser->id(), '<>');
        break;
    }
    return $query;
  }

}
