<?php

namespace Drupal\mcapi_signatures\Plugin\Block;
use Drupal\mcapi\Plugin\Block\WalletContextBlockBase;
use Drupal\user\Entity\User;
use Drupal\user\UserInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Entity\ContentEntityInterface;

/**
 * Provides a block for switching users.
 *
 * @Block(
 *   id = "mcapi_pending",
 *   admin_label = @Translation("Pending transactions"),
 *   category = "Community accounting"
 * )
 * @todo inject ModuleHandler
 */
class Pending extends WalletContextBlockBase implements ContainerFactoryPluginInterface {

  /**
   * {@inheritdoc}
   */
  public function label() {
    if ($this->configuration['holder_source'] == static::HOLDER_CURRENT_USER) {
      $label = $this->t('Pending transactions');
    }
    else {
      $owner = isset($this->holder) ? $this->getWalletOwner()->getDisplayName() : '[user]';
      $label = $this->t('Pending transactions for :name', [':name' => $owner]);
    }
    return $label;
  }

  private function getWalletOwner() : User {
    if ($this->holder instanceOf UserInterface) {
      $owner = $this->holder;
    }
    else {
      $owner = $this->holder->getOwner();
    }
    return $owner;
  }

  /**
   * {@inheritdoc}
   */
  public function build(): array {
    $renderable = [];
    $uid = $this->getWalletOwner()->id();
    $waiting_on_me = signature_needed_of_me($uid);
    $i_wait_for = signature_needed_of_other($uid);
    if ($serials = array_merge($waiting_on_me, $i_wait_for)) {
      rsort($serials);
      \Drupal::moduleHandler()->loadInclude('mcapi', 'inc');
      $renderable = mcapi_render_sentences($serials);
    }
    else {
      // @todo put an optional empty message
      $renderable[] = [
        '#markup' => t('No pending transactions.'),
        '#weight' => -1
      ];
    }
    return $renderable += parent::build();
  }

}
