<?php

namespace Drupal\mcapi\Plugin\Block;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Block\BlockBase;
use Drupal\mcapi\Entity\Currency;
use Drupal\Core\Render\Markup;

/**
 * Displays stats indicating the overall activity on the site.
 *
 * @Block(
 *   id = "mcapi_system_stats",
 *   admin_label = @Translation("System-wide trading stats"),
 *   category = @Translation("Community Accounting")
 * )
 */
class SystemStats extends BlockBase implements ContainerFactoryPluginInterface {


  public static function create($container, array $configuration, $plugin_id, $plugin_definition) {
    return new static ($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $currencies = Currency::loadMultiple();
    $currency = reset($currencies);

    $query = \Drupal::database()->select('mcapi_transactions_index', 't');
    $query->addExpression('COUNT(t.serial)', 'trades');
    $query->addExpression('COUNT(DISTINCT t.partner_id)', 'traders');
    $query->addExpression('sum(t.volume)', 'volume');
    $result = $query->condition('t.outgoing', 0)
      ->condition('t.curr_id', $currency->id)
      ->condition('created', strtotime('-365 days'), '>')
      ->execute()
      ->fetchAssoc();
    $build['#title'] = $this->t(':currency stats', [':currency' => $currency->label()]);
    $build['intro']['#markup'] = $this->t('Last 365 days...');
    $build['trades'] = $this->littleDiv($this->t('Trades'), $result['trades']);
    $build['traders'] = $this->littleDiv($this->t('Traders'), $result['traders']);
    $build['volume'] = $this->littleDiv($this->t('Volume'), $currency->format($result['volume']));
    return $build;
  }

  function littleDiv(string $title, string $value) {
    static $w = 0;
    $markup = '<strong>'.$title.':</strong> ';
    $markup .= $value;
    return [
      '#type' => 'markup',
      '#prefix' => '<div>',
      '#suffix' => '</div>',
      '#weight' => $w++,
      '#markup' => Markup::create($markup)
    ];
  }


}
